/*==================================================================================

  ITC - Integrated TurnOut Controller
  Version 1.0

------------------------------------------------------------------------------------
  This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License. 
  To view a copy of the license, visit http://creativecommons.org/licenses/by-sa/3.0/deed.en_US
------------------------------------------------------------------------------------

  Author:    Chuck Catania - Model Railroad Control Systems
  Project:   ITC
  Processor: Atmel AtTiny85
  Abstract:  This sketch implements a full suite of functions needed to
             to control a turnout motor on a model railroad layout.  The
             controller is base upon an AtTiny85 processor.
            
             The ITC takes a throw input from either a remote line or a 
             local input and sets the throw output.  An active low is the trigger level.
             
             UnLocked is an active low signal which will not allow the turnout to be thrown.
             If UnLocked is High, the Local Throw will throw the turnout.
            
             Feedback follows the state of the Throw signal.
   
             No point movement on lock/unlock operation.
             Turnout cannot be locked if not in Normal.

  Revision History:
   v1.0  02/07/2017  Production Release
   v0.10 12/07/2016  Renamed to ITC by popular demand
   v0.9  11/03/2016  Renamed to ITC Beta Version
   v0.8  10/24/2016  Implemented prototype lock/unlock
                     Implemented out of correspondence (OOC)
   v0.7  10/20/2016  Integrated with the demonstrator
   v0.6  10/18/2016  Corrected reset pin data value
                     Stubbed future out-of-correspondence code
   v0.5  10/11/2016  Alpha Version
   v0.1  08/08/2016  Initial Implementation
     
====================================================================================*/

//--------------------------------------------------------------------------
// AtTiny85 Pin Definitions
//--------------------------------------------------------------------------
byte       UL_OUT  =  PB0,  // PIN 5  Unlock Out
           TH_OUT  =  PB1,  // PIN 6  Throw Out  
           FB_OUT  =  PB2,  // PIN 7  Feedback Out 
           RTHW_IN =  PB3,  // PIN 2  Remote Throw In
           LTHW_IN =  PB4,  // PIN 3  Local Throw In
           UNLK_IN =  A0;   // PIN 1  Unlock In

const int  DEBOUNCE = 2;

const byte _NORMAL  = 0,
           _REVERSE = 1;
           
boolean  Locked = false;
         
byte     Remote_SW_pos   = 0,
         Remote_SW_state = 0,
         
         Local_SW_pos    = 0,
         Local_SW_state  = 0,
         
         prev_Locked_state  = 0,
         last_TO_State = _NORMAL,
         last_FB_State = _NORMAL;
         
unsigned long previousMS = 0,
              currentMS  = 0;

boolean    OOC = false,   // Out Of Correspondence flag
           Blink_OOC_LED = false,
           locked_State_Changed = false;
           
long       blinkMS   = 0,
           blinkRate = 0;
           
const int  blinkONrate  = 500,
           blinkOFFrate = 750;
          
/*************************************************************************/
/**********            GENERAL SUPPORT ROUTINES                 **********/
/*************************************************************************/
/*--------------------------------------------------------------------------
 Unlocked bit is connected to the Reset Pin. 
 The bit is treated as an analog voltage through a resistor voltage divider. 
 A value > 4v is a one, >= 2.5 a zero.
  
 With 1000 ohms for R1,R2, the analogRead of the reset pin returns
   1023 Switch Open    Unlock = FALSE
    509 Switch Closed  Unlock = TRUE  (Low)
    
 Set the Unlocked input based on the resistor divider voltage
---------------------------------------------------------------------------*/
boolean Read_Unlock( )
{
  if (analogRead( UNLK_IN ) > 600 ) 
   return false;
    
  return true;
   
}  // READ UNLOCK


/*---------------------------------------------------------------------
   Show the state of the Unlocked line
-----------------------------------------------------------------------*/
void Show_Unlock_State( ) 
{
  
  digitalWrite( UL_OUT, Locked); 
  
}  // SHOW UNLOCK STATE


/*---------------------------------------------------------------------
  Throw the turnout to the commanded state
-----------------------------------------------------------------------*/
void Throw_Turnout( byte to_State )
{

   digitalWrite( TH_OUT, to_State);  // Throw turnout
   last_TO_State = to_State;

   digitalWrite( FB_OUT, to_State);  // Set the Feedback state
   last_FB_State = to_State;
   
}  // THROW TURNOUT


/*---------------------------------------------------------------------
  Blink the Out Of Correspondence LED (Unlocked) if the local switch is
  not in Normal and the DS tries to lock the turnout
-----------------------------------------------------------------------*/
void Show_OOC()
{
  if (!OOC) return;
  
  if(currentMS - blinkMS > blinkRate)
   {      
     blinkMS = currentMS;                 // save the last time you blinked the LED      
     Blink_OOC_LED = (!Blink_OOC_LED);        // if the LED is off turn it on and vice-versa:     
     digitalWrite( UL_OUT, Blink_OOC_LED ); // set the LED with the ledState of the variable: 
 
     if (Blink_OOC_LED)
      blinkRate = blinkONrate;
     else
      blinkRate = blinkOFFrate;    
   }     
}  // SHOW OUT OF CORRESPONDENCE


/************************************************
*******            Setup Code          **********
*************************************************/
void setup()
{  
  pinMode( UNLK_IN, INPUT);        // Analog input voltage divider       
  pinMode( RTHW_IN, INPUT_PULLUP);   
  pinMode( LTHW_IN, INPUT_PULLUP);   
  
  pinMode( UL_OUT,  OUTPUT);   
  pinMode( TH_OUT,  OUTPUT);         
  pinMode( FB_OUT,  OUTPUT);   

  digitalWrite( TH_OUT, _NORMAL);  // Throw turnout
  digitalWrite( FB_OUT, _NORMAL);  // Set the Feedback state
  
// Read the initial state of the inputs.  
// All inputs are internally pulled up
//--------------------------------------
  Remote_SW_pos = (!digitalRead( RTHW_IN ));  // read the Remote switch throw input
  Remote_SW_state = Remote_SW_pos;

  Local_SW_pos = (!digitalRead( LTHW_IN ));   // read the Local switch throw input
  Local_SW_state = Local_SW_pos;

}  // SETUP


/**************************************************
*******          Main Loop Code          **********
***************************************************/
void loop()
{
  byte i;
  
  currentMS = millis();
  
  for (i=0; i<2; i++)
  {
    // Only allow switch lock operations to be done if turnout Normal
    //---------------------------------------------------------------
    Locked = Read_Unlock();                     // Read the Unlock line
    Remote_SW_pos = (!digitalRead( RTHW_IN ));  // Read the Remote switch lever
    Local_SW_pos = (!digitalRead( LTHW_IN ));   // Read the Lock switch lever
    delay(DEBOUNCE);
  }
  
// Check for locked state change
//------------------------------------
  if (Locked != prev_Locked_state)
   {
     prev_Locked_state = Locked; 
     locked_State_Changed = true;
     Show_Unlock_State( );  
   }

// Clear the OOC flag if everything is normal
//-------------------------------------------
  if ((Locked) && (last_TO_State == _NORMAL) && (Local_SW_pos == _NORMAL))
   OOC = false;

// Check for Out Of Correspondence in Local mode
//----------------------------------------------
  if (locked_State_Changed && (Locked) && (Local_SW_pos == _REVERSE) && (Remote_SW_pos == _NORMAL))
   {
     if (!OOC)
      {
       OOC = true;
       Locked = false;
      }
    }
   
// Throw the enabled turnout lever if the lock lever position has not changed
// Test for the lever position has changed and only throw if it has
//----------------------------------------------------------------------------
  if (!locked_State_Changed)
   switch (Locked)
   {
     // Switch is locked, only respond to a remote switch command
     //----------------------------------------------------------
     case true: 
      if (Remote_SW_state != Remote_SW_pos)
       {
        Remote_SW_state = Remote_SW_pos;
        Throw_Turnout( Remote_SW_state );
       }
     break;  
     
     // Switch is unlocked, only respond to a local switch command
     //-----------------------------------------------------------
     case false:     
      if (Local_SW_state != Local_SW_pos)
       {
        Local_SW_state = Local_SW_pos; 
        Throw_Turnout( Local_SW_state );
       }      
     break;
     
     default: ;  
   }
   
// Clear the locked state changed flag
//------------------------------------
  locked_State_Changed = false;
  
// Blink the Unlocked LED if out of correspondence
//------------------------------------------------
  Show_OOC();
   
}  // LOOP

